unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, uCiaServiceTools, Menus, uCiaTrayIcon;

type
  TMain = class(TForm)
    PopupMenu1: TPopupMenu;
    Show1: TMenuItem;
    Hide1: TMenuItem;
    Stopservice1: TMenuItem;
    procedure Stopservice1Click(Sender: TObject);
    procedure Show1Click(Sender: TObject);
    procedure Hide1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure TrayTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    TrayIcon: TCiaTray;
    TrayTimer: TTimer;
    procedure CreateTrayIcon;
    procedure CreateTrayTimer;
    procedure WMQueryEndSession(var Msg: TMessage); message WM_QUERYENDSESSION;
    procedure WMEndSession(var Msg: TMessage); message WM_ENDSESSION;
    procedure LogOffUser;
    procedure EndWindows;
  public
  end;

var
  Main: TMain;

implementation

{$R *.DFM}

procedure TMain.WMQueryEndSession(var Msg: TMessage);
begin
   Msg.Result := 1;
end;

procedure TMain.WMEndSession(var Msg: TMessage);
begin
   if Msg.WParam <> 0 then begin
      if Msg.LParam <> 0 then
         LogOffUser
      else
         EndWindows;
   end;
end;

procedure TMain.CreateTrayIcon;
begin
   TrayIcon := TCiaTray.Create(Application);
   TrayIcon.PopupMenu := PopupMenu1;
end;

procedure TMain.CreateTrayTimer;
begin
   TrayTimer := TTimer.Create(Application);
   TrayTimer.OnTimer  := TrayTimerTimer;
   TrayTimer.Interval := 120000;
   TrayTimer.Enabled  := True
end;

procedure TMain.FormCreate(Sender: TObject);
begin
   {  If we run as a service the we create a timer to check now and then to see
      if a user has logged on. If we run as normal application then we enable a
      tray icon. Note that you can use any tray icon. It dont have to be mine.
      Also you can for example only create a tray icon when you run as service
      and make a normal modal form when you run as GUI. Question of imagination }

   if CiaIsService then
      CreateTrayTimer
   else
      CreateTrayIcon;
end;

procedure TMain.TrayTimerTimer(Sender: TObject);
begin
   {  Tristan Marlow <tristan@ctilogistics.com> came on the idea to check for
      existance of the program manager (start menu) to check if a user has
      logged on. When this is the case we show the tray icon }

   if FindWindow('progman', 'Program Manager') <> 0 then begin
      CreateTrayIcon;
      TrayTimer.Free;
      TrayTimer := nil;
   end;
end;

procedure TMain.LogOffUser;
begin
   {  A user is logging off from windows. Here you can do some cleanup code if
      nececary. Usefull it if the service only runs after logon. We check here
      also if the TrayIcon exists, if so the we destroy it. Then we create again
      a timer to check now and then if a new user has logged on to create again
      the tray icon. }

   if Assigned(TrayIcon) then begin
      TrayIcon.Destroy;
      TrayIcon := nil;
      CreateTrayTimer;
   end;
end;

procedure TMain.EndWindows;
begin
   {  Windows is shutting down. Here you can do some cleanup code if nececary.
      Usefull if the service runs after windows startup. }
end;

procedure TMain.Stopservice1Click(Sender: TObject);
begin
   if CiaIsService then
      PostThreadMessage(CiaService.ServiceThread.ThreadID, WM_QUIT, 0, 0)
   else
      Application.Terminate;
end;

procedure TMain.Show1Click(Sender: TObject);
begin
   if not Visible then
      Show;
end;

procedure TMain.Hide1Click(Sender: TObject);
begin
   if Visible then
      Hide;
end;

procedure TMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   if Visible then begin
      CanClose := False;
      Hide;
   end;
end;

end.
